@@ -1,24 +1,10 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
+ * Copyright (C) 2013 Canonical, Ltd
+ * Copyright (C) 2015 Ratchanan Srirattanamet
  *
- *  oFono - Open Source Telephony - RIL Modem Support
- *
- *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
- *  Copyright (C) 2013 Canonical, Ltd. All rights reserved.
- *  Copyright (C) 2015 Ratchanan Srirattanamet.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,11 +30,13 @@
 
 #include "simutil.h"
 #include "util.h"
-
-#include "rilmodem.h"
+#include "rilutil.h"
 
 #include <drivers/infineonmodem/infineon_constants.h>
 
+#define EF_STATUS_INVALIDATED 0
+#define EF_STATUS_VALID 1
+
 /* Number of passwords in EPINC response */
 #define MTK_EPINC_NUM_PASSWD 4
 
@@ -1481,7 +1469,6 @@ static void ril_sim_remove(struct ofono_sim *sim)
 }
 
 static const struct ofono_sim_driver driver = {
-	.name			= RILMODEM,
 	.probe			= ril_sim_probe,
 	.remove			= ril_sim_remove,
 	.read_file_info		= ril_sim_read_info,
@@ -1501,13 +1488,4 @@ static const struct ofono_sim_driver driver = {
 	.query_facility_lock    = ril_query_facility_lock,
 };
 
-void ril_sim_init(void)
-{
-	DBG("");
-	ofono_sim_driver_register(&driver);
-}
-
-void ril_sim_exit(void)
-{
-	ofono_sim_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(sim, rilmodem, &driver)
