@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2012  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2012  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -65,57 +51,105 @@
 #define GOBI_WMS	(1 << 2)
 #define GOBI_WDS	(1 << 3)
 #define GOBI_PDS	(1 << 4)
-#define GOBI_PBM	(1 << 5)
-#define GOBI_UIM	(1 << 6)
-#define GOBI_CAT	(1 << 7)
-#define GOBI_CAT_OLD	(1 << 8)
-#define GOBI_VOICE	(1 << 9)
-#define GOBI_WDA	(1 << 10)
+#define GOBI_UIM	(1 << 5)
+#define GOBI_VOICE	(1 << 6)
+#define GOBI_WDA	(1 << 7)
+
+struct service_request {
+	struct qmi_service **member;
+	uint32_t service_type;
+};
 
 struct gobi_data {
-	struct qmi_device *device;
+	struct qmi_qmux_device *device;
 	struct qmi_service *dms;
 	struct qmi_service *wda;
+	struct qmi_service *nas;
+	struct qmi_service *wds;
+	struct qmi_service *wms;
+	struct qmi_service *voice;
+	struct qmi_service *pds;
+	struct qmi_service *uim;
+	struct service_request service_requests[16];
+	int cur_service_request;
+	int num_service_requests;
 	unsigned long features;
 	unsigned int discover_attempts;
 	uint8_t oper_mode;
-	bool using_mux;
-	bool using_qmi_wwan_q;
 	int main_net_ifindex;
 	char main_net_name[IFNAMSIZ];
 	uint32_t max_aggregation_size;
 	uint32_t set_powered_id;
+	bool using_mux : 1;
+	bool using_qmi_wwan : 1;
+	bool using_qmi_wwan_q : 1;
 };
 
 static void gobi_debug(const char *str, void *user_data)
 {
 	const char *prefix = user_data;
 
-	ofono_info("%s%s", prefix, str);
+	ofono_debug("%s%s", prefix, str);
 }
 
+static void gobi_io_debug(const char *str, void *user_data)
+{
+	const char *prefix = user_data;
+
+	ofono_debug("%s%s", prefix, str);
+}
+
+/*
+ * Probe the modem.  The following modem properties are expected to be set
+ * in order to initialize the driver properly:
+ *
+ * NetworkInterface
+ *   The string that contains the 'main' network device.  This can be
+ *   "rmnet_ipa" on SoC systems, or "wwan0" for upstream linux systems.
+ *
+ * NetworkInterfaceIndex
+ *   The index of the main interface given by NetworkInterface
+ *
+ * NetworkInterfaceKernelDriver
+ *   The kernel driver that is being used by the main network device.  Certain
+ *   drivers such as 'qmi_wwan' or 'qmi_wwan_q' are treated specifically.
+ *
+ * Bus
+ *   The bus of the modem.  Values can be "usb", "embedded", or "pci"
+ */
 static int gobi_probe(struct ofono_modem *modem)
 {
 	struct gobi_data *data;
-	const char *kernel_driver;
+	const char *if_driver;
+	const char *ifname;
+	int ifindex;
+	const char *bus;
 
 	DBG("%p", modem);
 
+	if_driver = ofono_modem_get_string(modem,
+						"NetworkInterfaceKernelDriver");
+	ifname = ofono_modem_get_string(modem, "NetworkInterface");
+	ifindex = ofono_modem_get_integer(modem, "NetworkInterfaceIndex");
+	bus = ofono_modem_get_string(modem, "Bus");
+
+	DBG("net: %s[%s](%d) %s", ifname, if_driver, ifindex, bus);
+
+	if (!if_driver || !ifname || !ifindex || !bus)
+		return -EPROTO;
+
 	data = l_new(struct gobi_data, 1);
 
-	kernel_driver = ofono_modem_get_string(modem, "KernelDriver");
-	DBG("kernel_driver: %s", kernel_driver);
-
-	if (!strcmp(kernel_driver, "qmi_wwan_q"))
+	if (!strcmp(if_driver, "qmi_wwan_q"))
 		data->using_qmi_wwan_q = true;
+	else if (!strcmp(if_driver, "qmi_wwan"))
+		data->using_qmi_wwan = true;
 
 	data->main_net_ifindex =
 		ofono_modem_get_integer(modem, "NetworkInterfaceIndex");
 	l_strlcpy(data->main_net_name,
 			ofono_modem_get_string(modem, "NetworkInterface"),
 			sizeof(data->main_net_name));
-	DBG("net: %s (%d)", data->main_net_name, data->main_net_ifindex);
-
 	ofono_modem_set_data(modem, data);
 	ofono_modem_set_capabilities(modem, OFONO_MODEM_CAPABILITY_LTE);
 
@@ -129,6 +163,24 @@ static void cleanup_services(struct gobi_data *data)
 
 	qmi_service_free(data->wda);
 	data->wda = NULL;
+
+	qmi_service_free(data->nas);
+	data->nas = NULL;
+
+	qmi_service_free(data->wds);
+	data->wds = NULL;
+
+	qmi_service_free(data->wms);
+	data->wms = NULL;
+
+	qmi_service_free(data->voice);
+	data->voice = NULL;
+
+	qmi_service_free(data->pds);
+	data->pds = NULL;
+
+	qmi_service_free(data->uim);
+	data->uim = NULL;
 }
 
 static void gobi_remove(struct ofono_modem *modem)
@@ -146,11 +198,26 @@ static void gobi_remove(struct ofono_modem *modem)
 
 	cleanup_services(data);
 
-	qmi_device_free(data->device);
+	qmi_qmux_device_free(data->device);
 
 	l_free(data);
 }
 
+static void add_service_request(struct gobi_data *data,
+					struct qmi_service **member,
+					uint32_t service_type)
+{
+	struct service_request req = { .member = member,
+					.service_type = service_type };
+
+	if (data->num_service_requests == L_ARRAY_SIZE(data->service_requests)) {
+		ofono_error("No room to add service request");
+		return;
+	}
+
+	data->service_requests[data->num_service_requests++] = req;
+}
+
 static void shutdown_cb(void *user_data)
 {
 	struct ofono_modem *modem = user_data;
@@ -160,7 +227,7 @@ static void shutdown_cb(void *user_data)
 
 	data->discover_attempts = 0;
 
-	qmi_device_free(data->device);
+	qmi_qmux_device_free(data->device);
 	data->device = NULL;
 
 	ofono_modem_set_powered(modem, FALSE);
@@ -174,7 +241,7 @@ static void shutdown_device(struct ofono_modem *modem)
 
 	cleanup_services(data);
 
-	if (qmi_device_shutdown(data->device, shutdown_cb, modem, NULL) < 0)
+	if (qmi_qmux_device_shutdown(data->device, shutdown_cb, modem, NULL) < 0)
 		shutdown_cb(modem);
 }
 
@@ -275,12 +342,49 @@ error:
 	shutdown_device(modem);
 }
 
+static void setup_qmi_wwan(const char *interface, uint32_t llproto)
+{
+	char raw_ip;
+	char new_raw_ip;
+
+	if (l_sysctl_get_char(&raw_ip, "/sys/class/net/%s/qmi/raw_ip",
+				interface) < 0) {
+		DBG("Couldn't query raw_ip setting");
+		return;
+	}
+
+	if (raw_ip != 'Y' && raw_ip != 'N') {
+		DBG("Unexpected value: %c", raw_ip);
+		return;
+	}
+
+	switch (llproto) {
+	case QMI_WDA_DATA_LINK_PROTOCOL_802_3:
+		new_raw_ip = 'N';
+		break;
+	case QMI_WDA_DATA_LINK_PROTOCOL_RAW_IP:
+		new_raw_ip = 'Y';
+		break;
+	default:
+		DBG("Unknown WDA Link Protocol");
+		return;
+	}
+
+	DBG("raw_ip: %c, want: %c", raw_ip, new_raw_ip);
+
+	if (raw_ip == new_raw_ip)
+		return;
+
+	if (l_sysctl_set_char(new_raw_ip, "/sys/class/net/%s/qmi/raw_ip",
+				interface) < 0)
+		DBG("Fail to set raw_ip to %c", new_raw_ip);
+}
+
 static void get_data_format_cb(struct qmi_result *result, void *user_data)
 {
 	struct ofono_modem *modem = user_data;
 	struct gobi_data *data = ofono_modem_get_data(modem);
 	uint32_t llproto;
-	enum qmi_device_expected_data_format expected_llproto;
 
 	DBG("");
 
@@ -290,24 +394,11 @@ static void get_data_format_cb(struct qmi_result *result, void *user_data)
 	if (!qmi_result_get_uint32(result, QMI_WDA_LL_PROTOCOL, &llproto))
 		goto done;
 
-	expected_llproto = qmi_device_get_expected_data_format(data->device);
+	if (data->using_qmi_wwan) {
+		const char *interface =
+			ofono_modem_get_string(modem, "NetworkInterface");
 
-	if ((llproto == QMI_WDA_DATA_LINK_PROTOCOL_802_3) &&
-			(expected_llproto ==
-				QMI_DEVICE_EXPECTED_DATA_FORMAT_RAW_IP)) {
-		if (!qmi_device_set_expected_data_format(data->device,
-					QMI_DEVICE_EXPECTED_DATA_FORMAT_802_3))
-			DBG("Fail to set expected data to 802.3");
-		else
-			DBG("expected data set to 802.3");
-	} else if ((llproto == QMI_WDA_DATA_LINK_PROTOCOL_RAW_IP) &&
-			(expected_llproto ==
-				QMI_DEVICE_EXPECTED_DATA_FORMAT_802_3)) {
-		if (!qmi_device_set_expected_data_format(data->device,
-					QMI_DEVICE_EXPECTED_DATA_FORMAT_RAW_IP))
-			DBG("Fail to set expected data to raw-ip");
-		else
-			DBG("expected data set to raw-ip");
+		setup_qmi_wwan(interface, llproto);
 	}
 
 done:
@@ -344,111 +435,128 @@ error:
 	shutdown_device(modem);
 }
 
-static void create_dms_cb(struct qmi_service *service, void *user_data)
+static void request_service_cb(struct qmi_service *service, void *user_data)
 {
 	struct ofono_modem *modem = user_data;
 	struct gobi_data *data = ofono_modem_get_data(modem);
+	struct service_request *req =
+		&data->service_requests[data->cur_service_request];
 
 	DBG("");
 
 	if (!service)
 		goto error;
 
-	data->dms = service;
+	*req->member = service;
 
-	if (qmi_service_create(data->device, QMI_SERVICE_WDA,
-					create_wda_cb, modem, NULL))
+	data->cur_service_request += 1;
+	if (data->cur_service_request == data->num_service_requests) {
+		DBG("All services requested, proceeding to create WDA");
+
+		if (qmi_qmux_device_create_client(data->device, QMI_SERVICE_WDA,
+						create_wda_cb, modem, NULL))
+			return;
+
+		goto error;
+	}
+
+	req = &data->service_requests[data->cur_service_request];
+	DBG("Requesting: %u", req->service_type);
+
+	if (qmi_qmux_device_create_client(data->device, req->service_type,
+					request_service_cb, modem, NULL))
 		return;
 
 error:
 	shutdown_device(modem);
 }
 
-static void create_shared_dms(struct ofono_modem *modem)
-{
-	struct gobi_data *data = ofono_modem_get_data(modem);
-
-	qmi_service_create_shared(data->device, QMI_SERVICE_DMS,
-						create_dms_cb, modem, NULL);
-}
-
 static void discover_cb(void *user_data)
 {
 	struct ofono_modem *modem = user_data;
 	struct gobi_data *data = ofono_modem_get_data(modem);
-	uint16_t major, minor;
+	uint16_t major;
+	uint16_t minor;
 
 	DBG("");
 
-	if (qmi_device_has_service(data->device, QMI_SERVICE_DMS))
+	if (qmi_qmux_device_has_service(data->device, QMI_SERVICE_DMS))
 		data->features |= GOBI_DMS;
-	if (qmi_device_has_service(data->device, QMI_SERVICE_NAS))
+	if (qmi_qmux_device_has_service(data->device, QMI_SERVICE_NAS))
 		data->features |= GOBI_NAS;
-	if (qmi_device_has_service(data->device, QMI_SERVICE_WMS))
-		data->features |= GOBI_WMS;
-	if (qmi_device_has_service(data->device, QMI_SERVICE_WDS))
+	if (qmi_qmux_device_has_service(data->device, QMI_SERVICE_WDS))
 		data->features |= GOBI_WDS;
-	if (qmi_device_has_service(data->device, QMI_SERVICE_WDA))
+	if (qmi_qmux_device_has_service(data->device, QMI_SERVICE_WDA))
 		data->features |= GOBI_WDA;
-	if (qmi_device_has_service(data->device, QMI_SERVICE_PDS))
+	if (qmi_qmux_device_has_service(data->device, QMI_SERVICE_PDS))
 		data->features |= GOBI_PDS;
-	if (qmi_device_has_service(data->device, QMI_SERVICE_PBM))
-		data->features |= GOBI_PBM;
-	if (qmi_device_has_service(data->device, QMI_SERVICE_UIM))
+	if (qmi_qmux_device_has_service(data->device, QMI_SERVICE_UIM))
 		data->features |= GOBI_UIM;
-	if (qmi_device_has_service(data->device, QMI_SERVICE_CAT))
-		data->features |= GOBI_CAT;
-	if (qmi_device_get_service_version(data->device,
-				QMI_SERVICE_CAT_OLD, &major, &minor))
-		if (major > 0)
-			data->features |= GOBI_CAT_OLD;
-	if (qmi_device_has_service(data->device, QMI_SERVICE_VOICE))
-			data->features |= GOBI_VOICE;
+	if (qmi_qmux_device_has_service(data->device, QMI_SERVICE_VOICE))
+		data->features |= GOBI_VOICE;
+
+	if (qmi_qmux_device_get_service_version(data->device, QMI_SERVICE_WMS,
+						&major, &minor)) {
+		if (major < 1 || (major == 1 && minor < 2))
+			ofono_warn("unsupported WMS version: %u.%u, need: 1.2",
+					major, minor);
+		else
+			data->features |= GOBI_WMS;
+	}
 
 	if (!(data->features & GOBI_DMS)) {
 		if (++data->discover_attempts < 3 &&
-				!qmi_device_discover(data->device, discover_cb,
+				!qmi_qmux_device_discover(data->device,
+								discover_cb,
 								modem, NULL))
 			return;
 
-		shutdown_device(modem);
-		return;
+		goto error;
 	}
 
-	create_shared_dms(modem);
+	add_service_request(data, &data->dms, QMI_SERVICE_DMS);
+	if (data->features & GOBI_NAS)
+		add_service_request(data, &data->nas, QMI_SERVICE_NAS);
+	if (data->features & GOBI_WDS)
+		add_service_request(data, &data->wds, QMI_SERVICE_WDS);
+	if (data->features & GOBI_WMS)
+		add_service_request(data, &data->wms, QMI_SERVICE_WMS);
+	if (data->features & GOBI_VOICE)
+		add_service_request(data, &data->voice, QMI_SERVICE_VOICE);
+	if (data->features & GOBI_UIM)
+		add_service_request(data, &data->uim, QMI_SERVICE_UIM);
+
+	if (qmi_qmux_device_create_client(data->device, QMI_SERVICE_DMS,
+					request_service_cb, modem, NULL) > 0)
+		return;
+error:
+	shutdown_device(modem);
 }
 
 static int gobi_enable(struct ofono_modem *modem)
 {
 	struct gobi_data *data = ofono_modem_get_data(modem);
-	const char *kernel_driver;
+	const char *device;
 	int r;
 
 	DBG("%p", modem);
 
-	kernel_driver = ofono_modem_get_string(modem, "KernelDriver");
-	if (!kernel_driver)
+	device = ofono_modem_get_string(modem, "Device");
+	if (!device)
 		return -EINVAL;
 
-	if (!strcmp(kernel_driver, "qrtr"))
-		data->device = qmi_device_new_qrtr();
-	else {
-		const char *device;
-
-		device = ofono_modem_get_string(modem, "Device");
-		if (!device)
-			return -EINVAL;
-
-		data->device = qmi_device_new_qmux(device);
-	}
-
+	data->device = qmi_qmux_device_new(device);
 	if (!data->device)
 		return -EIO;
 
 	if (getenv("OFONO_QMI_DEBUG"))
-		qmi_device_set_debug(data->device, gobi_debug, "QMI: ");
+		qmi_qmux_device_set_debug(data->device, gobi_debug, "");
 
-	r = qmi_device_discover(data->device, discover_cb, modem, NULL);
+	if (getenv("OFONO_QMI_IO_DEBUG"))
+		qmi_qmux_device_set_io_debug(data->device,
+						gobi_io_debug, "QMI: ");
+
+	r = qmi_qmux_device_discover(data->device, discover_cb, modem, NULL);
 	if (!r)
 		return -EINPROGRESS;
 
@@ -619,29 +727,27 @@ static void gobi_set_online(struct ofono_modem *modem, ofono_bool_t online,
 static void gobi_pre_sim(struct ofono_modem *modem)
 {
 	struct gobi_data *data = ofono_modem_get_data(modem);
-	const char *sim_driver = NULL;
+	bool legacy = ofono_modem_get_boolean(modem, "ForceSimLegacy");
 
 	DBG("%p", modem);
 
-	ofono_devinfo_create(modem, 0, "qmimodem", data->device);
+	ofono_devinfo_create(modem, 0, "qmimodem", qmi_service_clone(data->dms));
 
-	if (data->features & GOBI_UIM)
-		sim_driver = "qmimodem";
-	else if (data->features & GOBI_DMS)
-		sim_driver = "qmimodem_legacy";
-
-	if (ofono_modem_get_boolean(modem, "ForceSimLegacy"))
-		sim_driver = "qmimodem_legacy";
-
-	if (sim_driver)
-		ofono_sim_create(modem, 0, sim_driver, data->device);
+	if ((data->features & GOBI_UIM) && !legacy)
+		ofono_sim_create(modem, 0, "qmimodem",
+						qmi_service_clone(data->dms),
+						qmi_service_clone(data->uim));
+	else /* DMS always available */
+		ofono_sim_create(modem, 0, "qmimodem_legacy",
+						qmi_service_clone(data->dms));
 
 	if (data->features & GOBI_VOICE)
-		ofono_voicecall_create(modem, 0, "qmimodem", data->device);
+		ofono_voicecall_create(modem, 0, "qmimodem",
+					qmi_service_clone(data->voice));
 
-	if (data->features & GOBI_PDS)
+	if (data->features & GOBI_PDS) /* exclusive use, no need to clone */
 		ofono_location_reporting_create(modem, 0, "qmimodem",
-							data->device);
+						l_steal_ptr(data->pds));
 }
 
 static void gobi_setup_gprs(struct ofono_modem *modem)
@@ -654,7 +760,9 @@ static void gobi_setup_gprs(struct ofono_modem *modem)
 	char buf[256];
 	int i;
 
-	gprs = ofono_gprs_create(modem, 0, "qmimodem", data->device);
+	gprs = ofono_gprs_create(modem, 0, "qmimodem",
+					qmi_service_clone(data->wds),
+					qmi_service_clone(data->nas));
 	if (!gprs) {
 		ofono_warn("Unable to create gprs for: %s",
 					ofono_modem_get_path(modem));
@@ -665,8 +773,8 @@ static void gobi_setup_gprs(struct ofono_modem *modem)
 	if (n_premux == 0) {
 		interface = ofono_modem_get_string(modem, "NetworkInterface");
 
-		gc = ofono_gprs_context_create(modem, 0, "qmimodem",
-							data->device);
+		gc = ofono_gprs_context_create(modem, 0, "qmimodem", -1,
+						qmi_service_clone(data->wds));
 		if (!gc) {
 			ofono_warn("Unable to create gprs-context for: %s",
 					ofono_modem_get_path(modem));
@@ -691,8 +799,8 @@ static void gobi_setup_gprs(struct ofono_modem *modem)
 		sprintf(buf, "PremuxInterface%dMuxId", i + 1);
 		mux_id = ofono_modem_get_integer(modem, buf);
 
-		gc = ofono_gprs_context_create(modem, mux_id, "qmimodem",
-							data->device);
+		gc = ofono_gprs_context_create(modem, 0, "qmimodem", mux_id,
+						qmi_service_clone(data->wds));
 
 		if (!gc) {
 			ofono_warn("gprs-context creation failed for [%d] %s",
@@ -714,21 +822,18 @@ static void gobi_post_sim(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
-	ofono_lte_create(modem, 0, "qmimodem", data->device);
-
-	if (data->features & GOBI_CAT)
-		ofono_stk_create(modem, 0, "qmimodem", data->device);
-	else if (data->features & GOBI_CAT_OLD)
-		ofono_stk_create(modem, 1, "qmimodem", data->device);
-
-	if (data->features & GOBI_PBM)
-		ofono_phonebook_create(modem, 0, "qmimodem", data->device);
+	if (data->features & GOBI_WDS)
+		ofono_lte_create(modem, 0, "qmimodem",
+					qmi_service_clone(data->wds));
 
 	if (data->features & GOBI_NAS)
-		ofono_radio_settings_create(modem, 0, "qmimodem", data->device);
+		ofono_radio_settings_create(modem, 0, "qmimodem",
+					qmi_service_clone(data->dms),
+					qmi_service_clone(data->nas));
 
 	if (data->features & GOBI_WMS)
-		ofono_sms_create(modem, 0, "qmimodem", data->device);
+		ofono_sms_create(modem, 0, "qmimodem",
+					qmi_service_clone(data->wms));
 
 	if ((data->features & GOBI_WMS) && (data->features & GOBI_UIM) &&
 			!ofono_modem_get_boolean(modem, "ForceSimLegacy")) {
@@ -750,16 +855,21 @@ static void gobi_post_online(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	if (data->features & GOBI_NAS) {
-		ofono_netreg_create(modem, 0, "qmimodem", data->device);
-		ofono_netmon_create(modem, 0, "qmimodem", data->device);
+		ofono_netreg_create(modem, 0, "qmimodem",
+						qmi_service_clone(data->nas));
+		ofono_netmon_create(modem, 0, "qmimodem",
+						qmi_service_clone(data->nas));
 	}
 
 	if (data->features & GOBI_VOICE) {
-		ofono_ussd_create(modem, 0, "qmimodem", data->device);
-		ofono_call_settings_create(modem, 0, "qmimodem", data->device);
-		ofono_call_barring_create(modem, 0, "qmimodem", data->device);
+		ofono_ussd_create(modem, 0, "qmimodem",
+						qmi_service_clone(data->voice));
+		ofono_call_settings_create(modem, 0, "qmimodem",
+						qmi_service_clone(data->voice));
+		ofono_call_barring_create(modem, 0, "qmimodem",
+						qmi_service_clone(data->voice));
 		ofono_call_forwarding_create(modem, 0, "qmimodem",
-						data->device);
+						qmi_service_clone(data->voice));
 	}
 }
 
