@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2011  ST-Ericsson AB
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2011  ST-Ericsson AB.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -305,7 +291,7 @@ error:
 	dbus_message_unref(message);
 }
 
-static gboolean property_changed(DBusConnection *connection,
+static gboolean property_changed(DBusConnection *conn,
 					DBusMessage *message, void *user_data)
 {
 	DBusMessageIter iter;
@@ -335,10 +321,10 @@ static gboolean property_changed(DBusConnection *connection,
 	return TRUE;
 }
 
-static void mgr_connect(DBusConnection *connection, void *user_data)
+static void mgr_connect(DBusConnection *conn, void *user_data)
 {
-	property_changed_watch = g_dbus_add_signal_watch(connection, NULL,
-						NULL,
+	property_changed_watch = g_dbus_add_signal_watch(conn,
+						MGR_SERVICE, NULL,
 						MGR_MODEM_INTERFACE,
 						PROPERTY_CHANGED,
 						property_changed,
@@ -346,10 +332,10 @@ static void mgr_connect(DBusConnection *connection, void *user_data)
 	get_modems();
 }
 
-static void mgr_disconnect(DBusConnection *connection, void *user_data)
+static void mgr_disconnect(DBusConnection *conn, void *user_data)
 {
 	g_hash_table_remove_all(modem_list);
-	g_dbus_remove_watch(connection, property_changed_watch);
+	g_dbus_remove_watch(conn, property_changed_watch);
 	property_changed_watch = 0;
 }
 
