@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,8 +23,6 @@
 
 #define CALL_SETTINGS_FLAG_CACHED 0x1
 
-static GSList *g_drivers = NULL;
-
 /* 27.007 Section 7.7 */
 enum clir_status {
 	CLIR_STATUS_NOT_PROVISIONED =		0,
@@ -477,9 +461,10 @@ static void cw_ss_set_callback(const struct ofono_error *error, void *data)
 	struct ofono_call_settings *cs = data;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		DBG("setting CW via SS failed");
+		DBG("setting CW via SS failed with error: %s",
+			telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&cs->pending,
-					__ofono_error_failed(cs->pending));
+			__ofono_error_from_error(error, cs->pending));
 
 		return;
 	}
@@ -614,9 +599,10 @@ static void clip_cnap_colp_colr_ss_query_cb(const struct ofono_error *error,
 	const char *value;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		DBG("Error occurred during ss control query");
+		DBG("SS control query failed with error: %s",
+			telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&cs->pending,
-					__ofono_error_failed(cs->pending));
+			__ofono_error_from_error(error, cs->pending));
 
 		return;
 	}
@@ -724,16 +710,17 @@ static void clir_ss_query_callback(const struct ofono_error *error,
 	const char *value;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		DBG("setting clir via SS failed");
+		DBG("clir query via SS failed with error: %s",
+					telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&cs->pending,
-					__ofono_error_failed(cs->pending));
+				__ofono_error_from_error(error, cs->pending));
 
 		return;
 	}
 
 	switch (network) {
 	case CLIR_STATUS_UNKNOWN:
-		value = "uknown";
+		value = "unknown";
 		break;
 
 	case CLIR_STATUS_PROVISIONED_PERMANENT:
@@ -772,9 +759,10 @@ static void clir_ss_set_callback(const struct ofono_error *error, void *data)
 	struct ofono_call_settings *cs = data;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		DBG("setting clir via SS failed");
+		DBG("setting clir via SS failed with error: %s",
+			telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&cs->pending,
-					__ofono_error_failed(cs->pending));
+			__ofono_error_from_error(error, cs->pending));
 
 		return;
 	}
@@ -1330,38 +1318,22 @@ static DBusMessage *cs_set_property(DBusConnection *conn, DBusMessage *msg,
 	return __ofono_error_invalid_args(msg);
 }
 
-static GDBusMethodTable cs_methods[] = {
-	{ "GetProperties",	"",	"a{sv}",	cs_get_properties,
-							G_DBUS_METHOD_FLAG_ASYNC },
-	{ "SetProperty",	"sv",	"",		cs_set_property,
-							G_DBUS_METHOD_FLAG_ASYNC },
+static const GDBusMethodTable cs_methods[] = {
+	{ GDBUS_ASYNC_METHOD("GetProperties",
+				NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
+				cs_get_properties) },
+	{ GDBUS_ASYNC_METHOD("SetProperty",
+			GDBUS_ARGS({ "property", "s" }, { "value", "v" }),
+			NULL, cs_set_property) },
 	{ }
 };
 
-static GDBusSignalTable cs_signals[] = {
-	{ "PropertyChanged",	"sv" },
+static const GDBusSignalTable cs_signals[] = {
+	{ GDBUS_SIGNAL("PropertyChanged",
+			GDBUS_ARGS({ "property", "s" }, { "value", "v" })) },
 	{ }
 };
 
-int ofono_call_settings_driver_register(const struct ofono_call_settings_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	if (d->probe == NULL)
-		return -EINVAL;
-
-	g_drivers = g_slist_prepend(g_drivers, (void *) d);
-
-	return 0;
-}
-
-void ofono_call_settings_driver_unregister(const struct ofono_call_settings_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	g_drivers = g_slist_remove(g_drivers, (void *) d);
-}
-
 static void call_settings_unregister(struct ofono_atom *atom)
 {
 	struct ofono_call_settings *cs = __ofono_atom_get_data(atom);
@@ -1394,46 +1366,14 @@ static void call_settings_remove(struct ofono_atom *atom)
 	g_free(cs);
 }
 
-struct ofono_call_settings *ofono_call_settings_create(struct ofono_modem *modem,
-							unsigned int vendor,
-							const char *driver,
-							void *data)
-{
-	struct ofono_call_settings *cs;
-	GSList *l;
-
-	if (driver == NULL)
-		return NULL;
-
-	cs = g_try_new0(struct ofono_call_settings, 1);
-
-	if (cs == NULL)
-		return NULL;
-
+OFONO_DEFINE_ATOM_CREATE(call_settings, OFONO_ATOM_TYPE_CALL_SETTINGS, {
 	/* Set all the settings to unknown state */
-	cs->clip = CLIP_STATUS_UNKNOWN;
-	cs->cnap = CNAP_STATUS_UNKNOWN;
-	cs->clir = CLIR_STATUS_UNKNOWN;
-	cs->colp = COLP_STATUS_UNKNOWN;
-	cs->colr = COLR_STATUS_UNKNOWN;
-	cs->atom = __ofono_modem_add_atom(modem, OFONO_ATOM_TYPE_CALL_SETTINGS,
-						call_settings_remove, cs);
-
-	for (l = g_drivers; l; l = l->next) {
-		const struct ofono_call_settings_driver *drv = l->data;
-
-		if (g_strcmp0(drv->name, driver))
-			continue;
-
-		if (drv->probe(cs, vendor, data) < 0)
-			continue;
-
-		cs->driver = drv;
-		break;
-	}
-
-	return cs;
-}
+	atom->clip = CLIP_STATUS_UNKNOWN;
+	atom->cnap = CNAP_STATUS_UNKNOWN;
+	atom->clir = CLIR_STATUS_UNKNOWN;
+	atom->colp = COLP_STATUS_UNKNOWN;
+	atom->colr = COLR_STATUS_UNKNOWN;
+})
 
 static void ussd_watch(struct ofono_atom *atom,
 			enum ofono_atom_watch_condition cond, void *data)
