@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -256,7 +242,9 @@ static void dun_ath_cb(GAtServer *server, GAtServerRequestType type,
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_SET:
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto error;
 
 		if (g_at_result_iter_next_number(&iter, &val) == FALSE)
 			goto error;
@@ -307,7 +295,9 @@ static void dun_ato_cb(GAtServer *server, GAtServerRequestType type,
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_SET:
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto error;
 
 		if (g_at_result_iter_next_number(&iter, &val) == FALSE)
 			goto error;
@@ -369,7 +359,7 @@ static void notify_deferred_indicators(GAtServer *server, void *user_data)
 {
 	struct ofono_emulator *em = user_data;
 	int i;
-	char buf[20];
+	char buf[64];
 	GSList *l;
 	struct indicator *ind;
 
@@ -470,7 +460,9 @@ static void brsf_cb(GAtServer *server, GAtServerRequestType type,
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_SET:
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto fail;
 
 		if (g_at_result_iter_next_number(&iter, &val) == FALSE)
 			goto fail;
@@ -599,7 +591,9 @@ static void cmer_cb(GAtServer *server, GAtServerRequestType type,
 		int val;
 
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto fail;
 
 		/* mode */
 		if (!g_at_result_iter_next_number_default(&iter, mode, &mode))
@@ -683,7 +677,9 @@ static void clip_cb(GAtServer *server, GAtServerRequestType type,
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_SET:
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto fail;
 
 		if (!g_at_result_iter_next_number(&iter, &val))
 			goto fail;
@@ -721,7 +717,9 @@ static void ccwa_cb(GAtServer *server, GAtServerRequestType type,
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_SET:
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto fail;
 
 		if (!g_at_result_iter_next_number(&iter, &val))
 			goto fail;
@@ -764,7 +762,9 @@ static void cmee_cb(GAtServer *server, GAtServerRequestType type,
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_SET:
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto fail;
 
 		if (g_at_result_iter_next_number(&iter, &val) == FALSE)
 			goto fail;
@@ -810,7 +810,9 @@ static void bia_cb(GAtServer *server, GAtServerRequestType type,
 		int val;
 
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto fail;
 
 		/* check validity of the request */
 		while (g_at_result_iter_next_number_default(&iter, 0, &val))
@@ -823,7 +825,9 @@ static void bia_cb(GAtServer *server, GAtServerRequestType type,
 
 		/* request is valid, update the indicator activation status */
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto fail;
 
 		for (l = em->indicators; l; l = l->next) {
 			struct indicator *ind = l->data;
@@ -877,7 +881,9 @@ static void bind_cb(GAtServer *server, GAtServerRequestType type,
 		int num_hf_indicators = 0;
 
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto fail;
 
 		/* check validity of the request */
 		while (num_hf_indicators < 20 &&
@@ -895,7 +901,9 @@ static void bind_cb(GAtServer *server, GAtServerRequestType type,
 
 		/* request is valid, update the indicator activation status */
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto fail;
 
 		while (g_at_result_iter_next_number(&iter, &hf_indicator))
 			ofono_info("HF supports indicator: 0x%04x",
@@ -926,7 +934,9 @@ static void biev_cb(GAtServer *server, GAtServerRequestType type,
 		int val;
 
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto fail;
 
 		if (g_at_result_iter_next_number(&iter, &hf_indicator) == FALSE)
 			goto fail;
@@ -980,7 +990,9 @@ static void bac_cb(GAtServer *server, GAtServerRequestType type,
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_SET:
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			goto fail;
 
 		/*
 		 * CVSD codec is mandatory and must come first.
@@ -1069,7 +1081,9 @@ static void bcs_cb(GAtServer *server, GAtServerRequestType type,
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_SET:
 		g_at_result_iter_init(&iter, result);
-		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next(&iter, ""))
+			break;
 
 		if (!g_at_result_iter_next_number(&iter, &val))
 			break;
@@ -1138,7 +1152,7 @@ static void emulator_add_indicator(struct ofono_emulator *em, const char* name,
 		return;
 	}
 
-	ind->name = g_strdup(name);
+	ind->name = l_strdup(name);
 	ind->min = min;
 	ind->max = max;
 	ind->value = dflt;
@@ -1163,7 +1177,7 @@ static void emulator_unregister(struct ofono_atom *atom)
 	for (l = em->indicators; l; l = l->next) {
 		struct indicator *ind = l->data;
 
-		g_free(ind->name);
+		l_free(ind->name);
 		g_free(ind);
 	}
 
@@ -1347,6 +1361,7 @@ void ofono_emulator_send_final(struct ofono_emulator *em,
 	case OFONO_ERROR_TYPE_CEER:
 	case OFONO_ERROR_TYPE_SIM:
 	case OFONO_ERROR_TYPE_FAILURE:
+	case OFONO_ERROR_TYPE_ERRNO:
 failure:
 		g_at_server_send_final(em->server, G_AT_SERVER_RESULT_ERROR);
 		break;
@@ -1699,7 +1714,8 @@ int ofono_emulator_start_codec_negotiation(struct ofono_emulator *em,
 		 * Report we're done even if we don't have done any
 		 * negotiation as the other side may have to clean up.
 		 */
-		cb(0, data);
+		if (cb)
+			cb(0, data);
 
 		/*
 		 * If we didn't received any +BAC during the SLC setup the
