@@ -32,16 +32,15 @@
 
 struct test_info {
 	int service_fds[TEST_SERVICE_COUNT];
-	struct qmi_device *device;
+	struct qmi_qrtr_node *node;
 	struct l_timeout *timeout;
-	struct l_queue *services;
 
 	/* Data sent to our test service */
 	struct sockaddr_qrtr sender;
 	size_t received_len;
 	void *received;
 
-	bool discovery_callback_called		: 1;
+	bool lookup_callback_called		: 1;
 	bool service_send_callback_called	: 1;
 	bool internal_timeout_callback_called	: 1;
 	bool notify_callback_called		: 1;
@@ -153,13 +152,12 @@ static struct test_info *test_setup(void)
 
 	info = l_new(struct test_info, 1);
 	setup_test_qrtr_services(info);
-	info->device = qmi_device_new_qrtr();
-	assert(info->device);
+	info->node = qmi_qrtr_node_new(0);
+	assert(info->node);
 
 	/* Enable ofono logging */
-	qmi_device_set_debug(info->device, debug_log, NULL);
+	qmi_qrtr_node_set_debug(info->node, debug_log, NULL);
 
-	info->services = l_queue_new();
 	info->timeout = l_timeout_create(TEST_TIMEOUT, test_timeout_cb, info,
 								NULL);
 
@@ -172,9 +170,7 @@ static void test_cleanup(struct test_info *info)
 
 	l_free(info->received);
 	l_timeout_remove(info->timeout);
-	l_queue_destroy(info->services,
-				(l_queue_destroy_func_t) qmi_service_free);
-	qmi_device_free(info->device);
+	qmi_qrtr_node_free(info->node);
 
 	/* The qrtr services will be destroyed automatically. */
 	for (i = 0; i < TEST_SERVICE_COUNT; ++i)
@@ -185,42 +181,33 @@ static void test_cleanup(struct test_info *info)
 	l_main_exit();
 }
 
-static void test_create_qrtr_device(const void *data)
-{
-	struct test_info *info = test_setup();
+static void _l_test_cleanup_cleanup(void *p) { test_cleanup(*(void **) p); }
 
-	test_cleanup(info);
+static void test_create_qrtr_node(const void *data)
+{
+	_auto_(test_cleanup) struct test_info *info = test_setup();
 }
 
-static void discovery_complete_cb(void *user_data)
+static void lookup_complete_cb(void *user_data)
 {
 	struct test_info *info = user_data;
 
-	info->discovery_callback_called = true;
+	info->lookup_callback_called = true;
 }
 
-static void perform_discovery(struct test_info *info)
+static void perform_lookup(struct test_info *info)
 {
-	qmi_device_discover(info->device, discovery_complete_cb, info, NULL);
+	qmi_qrtr_node_lookup(info->node, lookup_complete_cb, info, NULL);
 
-	while (!info->discovery_callback_called)
+	while (!info->lookup_callback_called)
 		l_main_iterate(-1);
 }
 
-static void test_discovery(const void *data)
+static void test_lookup(const void *data)
 {
-	struct test_info *info = test_setup();
+	_auto_(test_cleanup) struct test_info *info = test_setup();
 
-	perform_discovery(info);
-
-	test_cleanup(info);
-}
-
-static void create_service_cb(struct qmi_service *service, void *user_data)
-{
-	struct test_info *info = user_data;
-
-	l_queue_push_tail(info->services, service);
+	perform_lookup(info);
 }
 
 /* Callbacks could queue other callbacks so continue until there are no more. */
@@ -234,29 +221,23 @@ static void perform_all_pending_work(void)
 
 static void test_create_services(const void *data)
 {
-	struct test_info *info = test_setup();
+	_auto_(test_cleanup) struct test_info *info = test_setup();
 	struct qmi_service *services[3];
 	uint32_t service_type;
 	size_t i;
 
-	perform_discovery(info);
+	perform_lookup(info);
 
 	for (i = 0; i < TEST_SERVICE_COUNT; i++) {
 		struct qmi_service *service;
-		uint16_t major, minor;
+		uint8_t version;
 
 		service_type = unique_service_type(i);
-		assert(qmi_service_create_shared(info->device, service_type,
-						create_service_cb, info, NULL));
-		perform_all_pending_work();
-
-		assert(l_queue_length(info->services) == 1);
-		service = l_queue_pop_head(info->services);
+		service = qmi_qrtr_node_get_service(info->node, service_type);
 		assert(service);
 
-		assert(qmi_service_get_version(service, &major, &minor));
-		assert(major == unique_service_version(i));
-		assert(minor == 0);
+		assert(qmi_service_get_version(service, &version));
+		assert(version == unique_service_version(i));
 
 		qmi_service_free(service);
 	}
@@ -266,28 +247,19 @@ static void test_create_services(const void *data)
 	 * call the callback.
 	 */
 	service_type = unique_service_type(TEST_SERVICE_COUNT);
-	assert(!qmi_service_create_shared(info->device, service_type,
-					create_service_cb, info, NULL));
-	perform_all_pending_work();
-	assert(l_queue_isempty(info->services));
+	assert(!qmi_qrtr_node_get_service(info->node, service_type));
 
 	/* Confirm that multiple services may be created for the same type */
 	service_type = unique_service_type(0);
 
 	for (i = 0; i < L_ARRAY_SIZE(services); i++) {
-		assert(qmi_service_create_shared(info->device, service_type,
-						create_service_cb, info, NULL));
-		perform_all_pending_work();
-
-		assert(l_queue_length(info->services) == 1);
-		services[i] = l_queue_pop_head(info->services);
+		services[i] = qmi_qrtr_node_get_service(info->node,
+								service_type);
 		assert(services[i]);
 	}
 
 	for (i = 0; i < L_ARRAY_SIZE(services); i++)
 		qmi_service_free(services[i]);
-
-	test_cleanup(info);
 }
 
 static bool received_data(struct l_io *io, void *user_data)
@@ -441,40 +413,53 @@ static void send_response_to_client(struct test_info *info, struct l_io *io)
 		l_main_iterate(-1);
 }
 
+static struct qmi_service *service_common_setup(struct test_info *info,
+								bool dedicated)
+{
+	uint32_t service_type;
+	struct qmi_service *service;
+
+	perform_lookup(info);
+
+	service_type = unique_service_type(0); /* Use the first service */
+
+	if (dedicated)
+		service = qmi_qrtr_node_get_dedicated_service(info->node,
+								service_type);
+	else
+		service = qmi_qrtr_node_get_service(info->node, service_type);
+
+	assert(service);
+
+	return service;
+}
+
+static struct l_io *setup_io(int fd, void *user_data)
+{
+	struct l_io *io = l_io_new(fd);
+
+	assert(io);
+	l_io_set_read_handler(io, received_data, user_data, NULL);
+	return io;
+}
+
 /*
  * Initiates a send of the TLV data payload to the test service. The test
  * service will respond with the same data payload.
  */
 static void test_send_data(const void *data)
 {
-	struct test_info *info = test_setup();
-	struct l_io *io;
-	uint32_t service_type;
-	struct qmi_service *service;
-
-	perform_discovery(info);
-
-	service_type = unique_service_type(0); /* Use the first service */
-	assert(qmi_service_create_shared(info->device, service_type,
-					create_service_cb, info, NULL));
-	perform_all_pending_work();
-	service = l_queue_pop_head(info->services);
-	assert(service);
-
-	io = l_io_new(info->service_fds[0]);
-	assert(io);
-	l_io_set_read_handler(io, received_data, info, NULL);
+	_auto_(test_cleanup) struct test_info *info = test_setup();
+	struct l_io *io = setup_io(info->service_fds[0], info);
+	struct qmi_service *service = service_common_setup(info, false);
 
 	send_request_via_qmi(info, service);
 	send_response_to_client(info, io);
 
 	l_io_destroy(io);
 	qmi_service_free(service);
-
-	test_cleanup(info);
 }
 
-
 static void notify_cb(struct qmi_result *result, void *user_data)
 {
 	struct test_info *info = user_data;
@@ -496,25 +481,11 @@ static void internal_timeout_cb(struct l_timeout *timeout, void *user_data)
 
 static void test_notifications(const void *data)
 {
-	struct test_info *info = test_setup();
-	struct l_io *io;
-	uint32_t service_type;
-	struct qmi_service *service;
+	_auto_(test_cleanup) struct test_info *info = test_setup();
+	struct qmi_service *service = service_common_setup(info, false);
+	struct l_io *io = setup_io(info->service_fds[0], info);
 	struct l_timeout *receive_timeout;
 
-	perform_discovery(info);
-
-	service_type = unique_service_type(0); /* Use the first service */
-	assert(qmi_service_create_shared(info->device, service_type,
-					create_service_cb, info, NULL));
-	perform_all_pending_work();
-	service = l_queue_pop_head(info->services);
-	assert(service);
-
-	io = l_io_new(info->service_fds[0]);
-	assert(io);
-	l_io_set_read_handler(io, received_data, info, NULL);
-
 	send_request_via_qmi(info, service);
 	send_response_to_client(info, io);
 
@@ -547,30 +518,25 @@ static void test_notifications(const void *data)
 	l_timeout_remove(receive_timeout);
 
 	l_io_destroy(io);
-	test_cleanup(info);
 }
 
 static void test_service_notification_independence(const void *data)
 {
-	struct test_info *info = test_setup();
+	_auto_(test_cleanup) struct test_info *info = test_setup();
 	struct l_io *io;
 	uint32_t service_type;
 	struct qmi_service *services[2];
 	size_t i;
 
-	perform_discovery(info);
+	perform_lookup(info);
 
 	service_type = unique_service_type(0); /* Use the first service */
 
-	io = l_io_new(info->service_fds[0]);
-	assert(io);
-	l_io_set_read_handler(io, received_data, info, NULL);
+	io = setup_io(info->service_fds[0], info);
 
 	for (i = 0; i < L_ARRAY_SIZE(services); i++) {
-		assert(qmi_service_create_shared(info->device, service_type,
-						create_service_cb, info, NULL));
-		perform_all_pending_work();
-		services[i] = l_queue_pop_head(info->services);
+		services[i] = qmi_qrtr_node_get_service(info->node,
+								service_type);
 		assert(services[i]);
 
 		send_request_via_qmi(info, services[i]);
@@ -595,7 +561,19 @@ static void test_service_notification_independence(const void *data)
 		qmi_service_free(services[i]);
 
 	l_io_destroy(io);
-	test_cleanup(info);
+}
+
+static void test_dedicated(const void *data)
+{
+	_auto_(test_cleanup) struct test_info *info = test_setup();
+	struct qmi_service *service = service_common_setup(info, true);
+	struct l_io *io = setup_io(info->service_fds[0], info);
+
+	send_request_via_qmi(info, service);
+	send_response_to_client(info, io);
+
+	l_io_destroy(io);
+	qmi_service_free(service);
 }
 
 static void exit_if_qrtr_not_supported(void)
@@ -623,13 +601,14 @@ int main(int argc, char **argv)
 	__ofono_log_init(argv[0], "*", FALSE);
 
 	l_test_init(&argc, &argv);
-	l_test_add("QRTR device creation", test_create_qrtr_device, NULL);
-	l_test_add("QRTR discovery", test_discovery, NULL);
+	l_test_add("QRTR node creation", test_create_qrtr_node, NULL);
+	l_test_add("QRTR lookup", test_lookup, NULL);
 	l_test_add("QRTR services may be created", test_create_services, NULL);
 	l_test_add("QRTR service sends/responses", test_send_data, NULL);
 	l_test_add("QRTR notifications", test_notifications, NULL);
 	l_test_add("QRTR service notifications are independent",
 				test_service_notification_independence, NULL);
+	l_test_add("QRTR dedicated service", test_dedicated, NULL);
 	result = l_test_run();
 
 	__ofono_log_cleanup();
